function [F,M]  = compute_forces_moments(x,delta,Va,alpha,beta,P,stall,statespace)

u = x(4);
v = x(5);
w = x(6);
phi = x(7);
theta = x(8);
psi = x(9);
p = x(10);
q = x(11);
r = x(12);

delta_e = delta(1);
delta_a = delta(2);
delta_t = delta(3);
    
if isequal(statespace,'full')
    D = dragforce(Va,alpha,beta,q,delta_e,P,stall);
    Y = sideforce(Va,beta,p,r,delta_a,P);
    L = liftforce(Va,alpha,q,delta_e,P,stall);
    l = rollmoment(Va,beta,p,r,delta_a,P);
    m = pitchmoment(Va,alpha,q,delta_e,P);
    n = yawmoment(Va,beta,p,r,delta_a,P);
    
elseif isequal(statespace,'lateral')
    D = 0;
    Y = sideforce(Va,beta,p,r,delta_a,P);
    L = 0;
    l = rollmoment(Va,beta,p,r,delta_a,P);
    m = 0;
    n = yawmoment(Va,beta,p,r,delta_a,P);
    
elseif isequal(statespace,'longitudinal')
    D = dragforce(Va,alpha,beta,q,delta_e,P,stall);
    Y = 0;
    L = liftforce(Va,alpha,q,delta_e,P,stall);
    l = 0;
    m = pitchmoment(Va,alpha,q,delta_e,P);
    n = 0;
else
    disp('Not a state-space')
end
    
f_x = [cos(alpha), -sin(alpha)]*[-D, -L]';
f_y = Y;
f_z = [sin(alpha), cos(alpha)]*[-D, -L]';

fa = [f_x,f_y,f_z]';
ma = [l,m,n]';


% Thrust forces and moments in BODY
if isequal(statespace,'full') || isequal(statespace,'longitudinal')
    fp = thrustforce(Va,delta_t,P);
    mp = thrustmoment(delta_t,P);
else 
    fp = 0;
    mp = 0;
end

% Gravity fore in BODY
fg = gravityforce(phi,theta,P);

F = fa + fp + fg;
M = ma + mp;

end