function xdot_lat = eom_lateral(x,delta,u,w,phi,theta,q,P)

% States = [v p r phi psi]
% Inputs = [delta_a]

% u,w,theta,q are longitudinal states, assumed constant (using trim values)

v = x(1);
p = x(2);
r = x(3);
delta_a = delta;

Va = sqrt(u^2 + v^2 + w^2);
alpha = atan2(w,u);              
beta = asin(v/Va);

D = 0;
Y = sideforce(Va,beta,p,r,delta_a,P);
L = 0;
l = rollmoment(Va,beta,p,r,delta_a,P);
m = 0;
n = yawmoment(Va,beta,p,r,delta_a,P);
    
f_x = [cos(alpha), -sin(alpha)]*[-D, -L]';
f_y = Y;
f_z = [sin(alpha), cos(alpha)]*[-D, -L]';

fa = [f_x,f_y,f_z]';
ma = [l,m,n]';

fp = 0;
mp = 0;

fg = gravityforce(phi,theta,P);

F = fa + fp + fg;
M = ma + mp;

X = F(1);
Y = F(2);
Z = F(3);
l = M(1);
m = M(2);
n = M(3);

vel_dot = [r*v - q*w;
           p*w - r*u;
           q*u - p*v] + 1/P.m * [X, Y, Z]';

angrate_dot = [P.L1*p*q - P.L2*q*r;
               P.L5*p*r - P.L6*(p^2-r^2);
               P.L7*p*q-P.L1*q*r] + ... 
               [P.L3*l + P.L4*n;
               (1/P.Jy)*m;
               P.L4*l + P.L8*n];
           
vdot = vel_dot(2);
pdot = angrate_dot(1);
rdot = angrate_dot(3);

% vdot = p*w-r*u+P.g*cos(theta)*sin(phi)+(P.rho*sqrt(u^2+v^2+w^2)*...
%     P.S_wing*P.b)/(4*P.m)*(P.C_Y_p*p + P.C_Y_r*r)...
%     + P.rho*(u^2+v^2+w^2)*P.S_wing/(2*P.m)*(P.C_Y_0+ ...
%     P.C_Y_beta*atan(v/sqrt(u^2+w^2))+P.C_Y_delta_a*delta_a);
% pdot = P.L1*p*q-P.L2*q*r+(P.rho*sqrt(u^2+v^2+w^2)*P.S_wing*(P.b)^2)/4*...
%     (P.C_p_p*p + P.C_p_r*r)+(1/2)*P.rho*(u^2+v^2+w^2)*P.S_wing*P.b*...
%     (P.C_p_0+P.C_p_beta*atan(v/sqrt(u^2+w^2))+P.C_p_delta_a*delta_a);
% rdot = P.L7*p*q-P.L1*q*r+P.rho*sqrt(u^2+v^2+w^2)*P.S_wing*(P.b)^2/4*...
%     (P.C_r_p*p + P.C_r_r*r)+(1/2)*P.rho*(u^2+v^2+w^2)*P.S_wing*P.b*...
%     (P.C_r_0+P.C_r_beta*atan(v/sqrt(u^2+w^2))+P.C_r_delta_a*delta_a);
           
xdot_lat = [vdot;pdot;rdot];
end