function xdot_lat = eom_lateral_sysid(x,delta,u,w,phi,theta,q,P,sysparam)

% States = [v p r]
% Inputs = [delta_a]

% u,w,theta,q are longitudinal states, assumed constant (using trim values)

C_Y_p = sysparam(1);
C_Y_r = sysparam(2);
C_Y_0 = sysparam(3);
C_Y_beta = sysparam(4);
C_Y_delta_a = sysparam(5);
C_l_0 = sysparam(6);
C_l_beta = sysparam(7);
C_l_p = sysparam(8);
C_l_r = sysparam(9);
C_l_delta_a = sysparam(10);
C_n_0 = sysparam(11);
C_n_beta = sysparam(12);
C_n_p = sysparam(13);
C_n_r = sysparam(14);
C_n_delta_a = sysparam(15);

v = x(1);
p = x(2);
r = x(3);
delta_a = delta;

Va = sqrt(u^2 + v^2 + w^2);
alpha = atan2(w,u);              
beta = asin(v/Va);

D = 0;
C_Y = (C_Y_0 + C_Y_beta*beta + (C_Y_p*P.b*p)/(2*Va)...
    + (C_Y_r*P.b*r)/(2*Va) + C_Y_delta_a*delta_a);
Y = 0.5*P.rho*Va^2*P.S_wing*C_Y;
L = 0;
C_l = C_l_0 + C_l_beta*beta + (C_l_p*P.b*p)/(2*Va)...
    + (C_l_r*P.b*r)/(2*Va) + C_l_delta_a*delta_a;
l = 0.5*P.rho*Va^2*P.S_wing*P.b*C_l;
m = 0;
C_n = C_n_0 + C_n_beta*beta + (C_n_p*P.b*p)/(2*Va)...
    + (C_n_r*P.b*r)/(2*Va) +...
    C_n_delta_a*delta_a;
n = 0.5*P.rho*Va^2*P.S_wing*P.b*C_n;
    
f_x = [cos(alpha), -sin(alpha)]*[-D, -L]';
f_y = Y;
f_z = [sin(alpha), cos(alpha)]*[-D, -L]';

fa = [f_x,f_y,f_z]';
ma = [l,m,n]';

fp = 0;
mp = 0;

fg = gravityforce(phi,theta,P);

F = fa + fp + fg;
M = ma + mp;

X = F(1);
Y = F(2);
Z = F(3);
l = M(1);
m = M(2);
n = M(3);

vel_dot = [r*v - q*w;
           p*w - r*u;
           q*u - p*v] + 1/P.m * [X, Y, Z]';

angrate_dot = [P.L1*p*q - P.L2*q*r;
               P.L5*p*r - P.L6*(p^2-r^2);
               P.L7*p*q-P.L1*q*r] + ... 
               [P.L3*l + P.L4*n;
               (1/P.Jy)*m;
               P.L4*l + P.L8*n];
           
vdot = vel_dot(2);
pdot = angrate_dot(1);
rdot = angrate_dot(3);

% vdot = p*w-r*u+P.g*cos(theta)*sin(phi)+(P.rho*sqrt(u^2+v^2+w^2)*...
%     P.S_wing*P.b)/(4*P.m)*(C_Y_p*p + C_Y_r*r)...
%     + P.rho*(u^2+v^2+w^2)*P.S_wing/(2*P.m)*(C_Y_0+ ...
%     C_Y_beta*atan(v/sqrt(u^2+w^2))+C_Y_delta_a*delta_a);
% pdot = P.L1*p*q-P.L2*q*r+(P.rho*sqrt(u^2+v^2+w^2)*P.S_wing*(P.b)^2)/4*...
%     (C_p_p*p + C_p_r*r)+(1/2)*P.rho*(u^2+v^2+w^2)*P.S_wing*P.b*...
%     (C_p_0+C_p_beta*atan(v/sqrt(u^2+w^2))+C_p_delta_a*delta_a);
% rdot = P.L7*p*q-P.L1*q*r+P.rho*sqrt(u^2+v^2+w^2)*P.S_wing*(P.b)^2/4*...
%     (C_r_p*p + C_r_r*r)+(1/2)*P.rho*(u^2+v^2+w^2)*P.S_wing*P.b*...
%     (C_r_0+C_r_beta*atan(v/sqrt(u^2+w^2))+C_r_delta_a*delta_a);
           
xdot_lat = [vdot;pdot;rdot];
end