function [x_dot]  = equationsOfMotion(x,delta,P,statespace)

u = x(4);
v = x(5);
w = x(6);
phi = x(7);
theta = x(8);
psi = x(9);
p = x(10);
q = x(11);
r = x(12);
Va = sqrt(u^2 + v^2 + w^2);
alpha = atan2(w,u);              
beta = asin(v/Va);

stall = false;
[F,M] = compute_forces_moments(x,delta,Va,alpha,beta,P,stall,statespace);

X = F(1);
Y = F(2);
Z = F(3);
l = M(1);
m = M(2);
n = M(3);

R_1 = Rzyx(phi,theta,psi);  % BODY to NED 
R_2 = [1, sin(phi)*tan(theta), cos(phi)*tan(theta);
   0, cos(phi), -sin(phi);
   0, sin(phi)*sec(theta), cos(phi)*sec(theta)]; % BODY to VEHICLE

% Kinematics (geometrical aspect of motion) 3.14 & 3.16 Beard
pos_dot = R_1*[u,v,w]';
ang_dot = R_2*[p,q,r]';
   
% Kinetics (forces causing the motion) 3.15 & 3.17 Beard
vel_dot = [r*v - q*w;
           p*w - r*u;
           q*u - p*v] + 1/P.m * [X, Y, Z]';

angrate_dot = [P.L1*p*q - P.L2*q*r;
               P.L5*p*r - P.L6*(p^2-r^2);
               P.L7*p*q-P.L1*q*r] + ... 
               [P.L3*l + P.L4*n;
               (1/P.Jy)*m;
               P.L4*l + P.L8*n];
           
if isequal(statespace,'full')
    x_dot = [pos_dot;vel_dot;ang_dot;angrate_dot];
elseif isequal(statespace,'lateral')
    x_dot = [vel_dot(2);angrate_dot(1);angrate_dot(3);ang_dot(1);ang_dot(3)];
elseif isequal(statespace,'longitudinal')
    x_dot = [vel_dot(1);vel_dot(3);angrate_dot(2);ang_dot(2);pos_dot(3)];
else
    disp('Not a state-space')
end

end