close all;
clear;
clc;

addpath('GNC')
addpath('forces')
% prompt = 'pc or mac? ';
% path = input(prompt,'s');
% if isequal(path,'pc')
%     addpath('C:\Users\tobiasva\Documents\Casadi')
% elseif isequal(path,'mac')
%     addpath('/Users/tobiasvaage/casadi-osx-matlabR2015a-v3.5.5')
% else
%     disp('Try again')
% end
import casadi.*
params;
warning('off','all')

statespace = 'full';
simtime = 30;

% Trim conditions and initialization
Va = 18;
Vg = Va;
R = Inf; % Must be nonzero
gamma = deg2rad(0);
x0 = [0 0 -100 Va 0 0 0 0 0 0 0 0 0 0 0.1];

% Compute trim and linearize
[A,B,C,D,x_star] = linearization(x0,Va,R,gamma,P,statespace);

Initial conditions for simulink model, and this vector changes
along with that model. See variable "xout" for correct x0_sim vector.
x0_sim = [0  x_star(4) x_star(5) x_star(6) 0 0 0 0 x_star(9) 0 x_star(7)...
    x_star(10) 0 0 -x_star(3) 0 0 x_star(3) 0 x_star(8) x_star(11) Va 0 ...
    0 0 x_star(2) x_star(1) x_star(12)]';

Vw = [0 0 0]';
u_r = x_star(4) - Vw(1);
v_r = x_star(5) - Vw(2);
w_r = x_star(6) - Vw(3);
Va_star = sqrt(u_r^2 + v_r^2 + w_r^2);
alpha_star = atan2(w_r,u_r);              
beta_star = asin(v_r/Va_star);

trim_info = struct('pn_star',x_star(1),'pe_star',x_star(2),'pd_star',x_star(3),...
    'u_star',x_star(4),'v_star',x_star(5),'w_star',x_star(6),'phi_star',x_star(7),...
    'theta_star',x_star(8),'psi_star',x_star(9),'p_star',x_star(10),...
    'q_star',x_star(11),'r_star',x_star(12),'delta_e_star',x_star(13),...
    'delta_a_star',x_star(14),'delta_t_star',x_star(15),'Va_star',Va_star,...
    'alpha_star',alpha_star,'beta_star',beta_star);
aTable = struct2table(trim_info);
disp(aTable);

% Necessary states for transfer functions
theta_star = x_star(8);
chi_star = x_star(9); % Assuming no wind
delta_e_star = x_star(13);
delta_a_star = x_star(14);
delta_t_star = x_star(15);

% Create buses for simulink
Trim = Simulink.Bus.createObject(trim_info);
Params = Simulink.Bus.createObject(P);
Simulink.Bus.createObject('full_x_bus_creator','full_x_bus_creator/Bus Creator')
Simulink.Bus.createObject('full_xdot_bus_creator','full_xdot_bus_creator/Bus Creator')
Simulink.Bus.createObject('full_u_bus_creator','full_u_bus_creator/Bus Creator')

% Waypoints
WP = [x_star(1) 1000 1500 2000; % pn
      x_star(2) 300 -200 200]; % pe

% Look up tables for Gain Scheduling
Gains = struct;
Gains.BP = [15 17 19 21 23 25]';
Gains.Kp_phi = [1.1667 1.75 1.75 2.3333 2.3333 3.5]';
Gains.Kd_phi = [0.0186 0.0394 0.0353 0.0476 0.0435 0.0756]';
Gains.Kp_chi = [7.1041 6.8282 8.5283 12.0314 14.4322 20.8835]';
Gains.Ki_chi = [10.1872 13.4565 18.7865 33.8206 44.4322 85.5924]';
Gains.Kp_theta = -[2.3333 2.3333 3.5 3.5 3.5 7]';
Gains.Kd_theta = -[0.2972 0.2622 0.2832 0.2562 0.2340 0.3014]';
Gains.Kp_h = [0.1998 0.1998 0.2228 0.2228 0.2228 0.2015]';
Gains.Ki_h = [0.2450 0.2776 0.4106 0.4538 0.4971 0.9454]';
Gains.Kp_v = [1.6484 1.6725 2.0046 2.0162 2.0236 3.3313]';
Gains.Ki_v = [1.4319 1.6457 2.6174 2.9098 3.1998 6.5189]';

%% Transfer functions

actuator_dynamics = tf(5,[1 5]);

% Aileron to roll
a_phi1 = -(1/2)*P.rho*Va^2*P.S_wing*P.b*P.C_p_p*P.b/(2*Va); % Need Va for this
a_phi2 = (1/2)*P.rho*Va^2*P.S_wing*P.b*P.C_p_delta_a; % Need Va for this
tf_aileron_roll = tf(a_phi2,[1,a_phi1]);

% Roll to course
tf_roll_course = tf(P.g/Vg,[1,0]); % Va or Vg, yaw or course

% Elevator to pitch
a_theta1 = -(P.rho*Va^2*P.c*P.S_wing*P.C_m_q*P.c)/(4*P.Jy*Va);
a_theta2 = -(P.rho*Va^2*P.c*P.S_wing*P.C_m_alpha)/(2*P.Jy);
a_theta3 = (P.rho*Va^2*P.c*P.S_wing*P.C_m_delta_e)/(2*P.Jy);
d_theta1 = 0;
d_theta2 = 0;
tf_elevator_pitch = tf([a_theta3,0],[1,a_theta1,a_theta2]);

% Pitch/Airspeed to altitude
d_h = 1;
tf_pitch_altitude = tf(Va,[1,0]); % Airspeed is constant
% tf_Va_altitude = tf([theta],[1,0]); % Pitch is constant

% Throttle/Pitch to airspeed
d_v1 = 0;
d_v2 = 0;
d_v = 0;
a_v1 = P.rho*Va_star*P.S_wing*(P.C_D_0+P.C_D_alpha1*alpha_star+...
    P.C_D_delta_e*delta_e_star)/P.m + P.rho*P.S_prop*P.C_prop*Va_star/P.m;
a_v2 = P.rho*P.S_wing*P.C_prop*P.k_motor^2*delta_t_star/P.m;
a_v3 = P.g*cos(theta_star-chi_star);
tf_throttle_airspeed = tf(a_v2,[1,a_v1]);
tf_pitch_airspeed = tf(-a_v3,[1,a_v1]);
tf_airspeed = tf(1,[1,a_v1]);

%% Controller gains
% Roll loop
zeta_phi = 0.9; % Design parameter
e_phi_max = deg2rad(20); % Design parameter
delta_a_max = P.delta_a_max;
omega_n_phi = sqrt(abs(a_phi2)*(delta_a_max/e_phi_max));
Kp_phi = (delta_a_max/e_phi_max)*sign(a_phi2);
Kd_phi = (2*zeta_phi*omega_n_phi - a_phi1)/a_phi2;
Ki_phi = -0.5; % Trial and error, use rlocus when ideally
% Questionable to use integral term in inner loop...

% Course loop
W_chi = 9; % bandwidth separation (design parameter)
zeta_chi = 0.707; % Design parameter
omega_n_chi = omega_n_phi/W_chi;
Kp_chi = 2*zeta_chi*omega_n_chi*Vg/P.g;
Ki_chi = omega_n_chi^2*Vg/P.g;

% Pitch loop
zeta_theta = 0.707; % Design parameter
e_theta_max = deg2rad(20); % Design parameter
delta_e_max = P.delta_e_max;
omega_n_theta = sqrt(a_theta2 + (delta_e_max/e_theta_max)*abs(a_theta3));
Kp_theta = (delta_e_max/e_theta_max)*sign(a_theta3);
Kd_theta = (2*zeta_theta*omega_n_theta - a_theta1)/a_theta3;
K_theta_DC = (Kp_theta*a_theta3)/(a_theta2 + Kp_theta*a_theta3);

% Altitude loop
W_h = 9; % Design parameter
zeta_h = 0.707; % Design parameter
omega_n_h = omega_n_theta/W_h;
Kp_h = 2*zeta_h*omega_n_h/(K_theta_DC*Va); % Va necessary
Ki_h = omega_n_h^2/(K_theta_DC*Va);        % Va necessary

% Airspeed loop
zeta_v = 9;
omega_n_v = omega_n_h*10;  % Have to compute this... or OK?
Kp_v = (2*zeta_v*omega_n_v - a_v1)/a_v2;
Ki_v = omega_n_v^2/a_v2;

%% Reference models

w_ref = 1.44;
w_ref_chi = w_ref; %omega_n_chi*0.5;
w_ref_h = w_ref; %omega_n_h*0.7;
w_ref_Va = w_ref; %omega_n_v*0.1;

Ad_chi = [0 1 0; 0 0 1; -w_ref_chi^3 -3*w_ref_chi^2 -3*w_ref_chi];
Ad_h = [0 1 0; 0 0 1; -w_ref_h^3 -3*w_ref_h^2 -3*w_ref_h];
Ad_Va = [0 1; -w_ref_Va^2 -2*w_ref_Va];
Bd_chi = [0;0;w_ref_chi^3];
Bd_h = [0;0;w_ref_h^3];
Bd_Va = [0;w_ref_Va^2];
Cd_chi = [1 0 0];
Cd_h = [1 0 0];
Cd_Va = [1 0];
Dd = 0;
x0_chi = [0 0 0];
x0_h = [100 0 0];
x0_Va = [Va 0];

sim('Autopilot',simtime)

%% Plot

% The xlines are for the case study, 
% and should be removed for normal simulation
% lat plot = 1:1000
% lon plot = 1000:1887
figure(1)
sgtitle('Lateral Control System')
subplot(211)
plot(chi.time,chi.signals.values,'b','Linewidth',1);
hold on
plot(chi.time,chi_c.signals.values,'g','Linewidth',1);
xlabel('$time$ $[s]$','Interpreter','latex');
ylabel('$[deg]$','Interpreter','latex');
% xline(10,'-',{'15 m/s'});
% xline(20,'-',{'17 m/s'});
% xline(30,'-',{'19 m/s'});
% xline(40,'-',{'21 m/s'});
% xline(50,'-',{'23 m/s'});
% xline(60,'-',{'25 m/s'});
legend('Course','Desired course','FontSize',14)

subplot(212)
plot(aileron.time,aileron.signals.values,'r','Linewidth',1);
xlabel('$time$ $[s]$','Interpreter','latex');
ylabel('$[deg]$','Interpreter','latex');
% xline(10,'-',{'15 m/s'});
% xline(20,'-',{'17 m/s'});
% xline(30,'-',{'19 m/s'});
% xline(40,'-',{'21 m/s'});
% xline(50,'-',{'23 m/s'});
% xline(60,'-',{'25 m/s'});
legend('Actuator, [\delta_a]','FontSize',14)

figure(2)
sgtitle('Longitudinal Control System')
subplot(411)
plot(h.time,h.signals.values,'b','Linewidth',1)
hold on
plot(h.time,h_c.signals.values,'g','Linewidth',1)
legend('Altitude','Desired altitude')
xlabel('$time$ $[s]$','Interpreter','latex')
ylabel('$[m]$','Interpreter','latex')
% xline(80,'-',{'15 m/s'});
% xline(90,'-',{'17 m/s'});
% xline(100,'-',{'19 m/s'});
% xline(110,'-',{'21 m/s'});
% xline(120,'-',{'23 m/s'});
% xline(130,'-',{'25 m/s'});
legend('Altitude','Desired altitude','FontSize',14)

subplot(412)
plot(elevator.time,elevator.signals.values,'r','Linewidth',1)
xlabel('$time$ $[s]$','Interpreter','latex')
ylabel('$[deg]$','Interpreter','latex')
% xline(80,'-',{'15 m/s'});
% xline(90,'-',{'17 m/s'});
% xline(100,'-',{'19 m/s'});
% xline(110,'-',{'21 m/s'});
% xline(120,'-',{'23 m/s'});
% xline(130,'-',{'25 m/s'});
legend('Actuator, [\delta_e]','FontSize',14)

subplot(413)
plot(Va.time,Va.signals.values,'b','Linewidth',1)
hold on
plot(Va_c.time,Va_c.signals.values,'g','Linewidth',1)
legend('Airspeed','Desired airspeed','FontSize',14)
xlabel('$time$ $[s]$','Interpreter','latex')
ylabel('$[m/s]$','Interpreter','latex')

subplot(414)
plot(thrust.time,thrust.signals.values,'r','Linewidth',1)
xlabel('$time$ $[s]$','Interpreter','latex')
ylabel('[%]','Interpreter','latex')
% xline(80,'-',{'15 m/s'});
% xline(90,'-',{'17 m/s'});
% xline(100,'-',{'19 m/s'});
% xline(110,'-',{'21 m/s'});
% xline(120,'-',{'23 m/s'});
% xline(130,'-',{'25 m/s'});
legend('Actuator, [\delta_t]','FontSize',14)


figure(3)
hold on
siz=size(WP);
for ii=1:(siz(2)-1)   

plot([WP(2,ii), WP(2,ii+1)], [WP(1,ii), WP(1,ii+1)],'r--x')

end
plot(y.signals.values,x.signals.values,'b'); 
axis('equal')
xlabel('East')
ylabel('North')
title('North-East positions (m)');

figure(4)
plot3(x.signals.values,y.signals.values,h.signals.values,'Color','r','Linewidth',2)
axis equal
xlabel('North')
ylabel('East')
zlabel('Altitude')
grid on
title('3D Flight path')

figure(5)
plot(Va_c.time,Va_c.signals.values,'Linewidth',1)
% yline(15,'--');
% yline(17,'--');
% yline(19,'--');
% yline(21,'--');
% yline(23,'--');
% yline(25,'--');
xlabel('$time$ $[s]$','Interpreter','latex')
ylabel('$[m/s]$','Interpreter','latex')
title('Airspeed reference')

