clear all;
clc;
addpath('GNC')
addpath('C:\Users\tobiasva\Documents\Casadi')
% addpath('/Users/tobiasvaage/casadi-osx-matlabR2015a-v3.5.5')
addpath('forces')
import casadi.*
params;

load('x_lon.mat','x_lon')
load('u_lon.mat','u_lon')
load('x0_lon.mat','x0_lon')

load('x_lat.mat','x_lat')
load('u_lat.mat','u_lat')
load('x0_lat.mat','x0_lat')

% In this example, we fit a nonlinear model to measurements
%
% This example uses more advanced constructs than the vdp* examples:
% Since the number of control intervals is potentially very large here,
% we use memory-efficient Map and MapAccum, in combination with
% codegeneration.
%
% We will be working with a 2-norm objective:
% || y_measured - y_simulated ||_2^2
%
% This form is well-suited for the Gauss-Newton Hessian approximation.

%%%%%%%%%%% SETTINGS %%%%%%%%%%%%%%%%%%%%%
N = 200;  % Number of samples
fs = 50; % Sampling frequency [hz]

t = 1/fs:1/fs:4;

% param_truth = [P.C_L_0,P.C_L_alpha,P.C_L_delta_e,P.C_L_q,P.C_D_0,P.C_D_alpha1,...
%     P.C_D_delta_e,P.C_D_q,P.C_m_alpha,P.C_m_delta_e,P.C_m_q,P.C_m_0]';
% param_guess = zeros(12,1);
% param_guess(1) = param_truth(1) + 0.1*rand(1);
% param_guess(2) = param_truth(2) + 10*rand(1);
% param_guess(3) = param_truth(3) + 1*rand(1);
% param_guess(4) = param_truth(4) + 10*rand(1);
% param_guess(5) = param_truth(5) + 0.1*rand(1);
% param_guess(6) = param_truth(6) + 0.1*rand(1);
% param_guess(7) = param_truth(7) + 0.1*rand(1);
% param_guess(8) = param_truth(8);
% param_guess(9) = param_truth(9) + 1*rand(1);
% param_guess(10) = param_truth(10) + 1*rand(1);
% param_guess(11) = param_truth(11) + 10*rand(1);
% param_guess(12) = param_truth(12) + 0.1*rand(1);


param_truth = [P.C_Y_p,P.C_Y_r,P.C_Y_0,P.C_Y_beta,P.C_Y_delta_a,P.C_l_0,...
    P.C_l_beta,P.C_l_p,P.C_l_r,P.C_l_delta_a,P.C_n_0,P.C_n_beta,...
    P.C_n_p,P.C_n_r,P.C_n_delta_a]';
param_guess = zeros(15,1);
param_guess(1) = param_truth(1) + 1*rand(1);
param_guess(2) = param_truth(2) + 0.1*rand(1);
param_guess(3) = param_truth(3) + 0.001*rand(1);
param_guess(4) = param_truth(4) + 1*rand(1);
param_guess(5) = param_truth(5) + 0.1*rand(1);
param_guess(6) = param_truth(6) + 0.01*rand(1);
param_guess(7) = param_truth(7) + 0.1*rand(1);
param_guess(8) = param_truth(8) + 1*rand(1);
param_guess(9) = param_truth(9) + 0.1*rand(1);
param_guess(10) = param_truth(10) + 1*rand(1);
param_guess(11) = param_truth(11) + 0.0001*rand(1);
param_guess(12) = param_truth(12) + 0.1*rand(1);
param_guess(13) = param_truth(13) + 0.01*rand(1);
param_guess(14) = param_truth(14) + 0.1*rand(1);
param_guess(15) = param_truth(15) + 0.01*rand(1);

scale = ones(size(param_guess));

% Initial conditions for trim
%     pn pe pd   u  v  w  phi theta psi p  q  r  e  a  t
x0 = [0  0  -100 18 0  0  0   0     0   0  0  0  0  0  0.1];

% Trim calculation
Va_d = 18;
R_d = Inf;
gamma_d = 0;
[x,~] = trim(Va_d, R_d, gamma_d, x0, P);

%%%%%%%%%%%% MODELING %%%%%%%%%%%%%%%%%%%%%

pn = MX.sym('pn');
pe = MX.sym('pe');
pd = MX.sym('pd');
u = MX.sym('u');
v = MX.sym('v');
w = MX.sym('w');
phi = MX.sym('phi');
theta = MX.sym('theta');
psi = MX.sym('psi');
p = MX.sym('p');
q = MX.sym('q');
r = MX.sym('r');

delta_e = MX.sym('delta_e');
delta_a = MX.sym('delta_a');
delta_t = MX.sym('delta_t');

% states = [pn;pe;pd;u;v;w;phi;theta;psi;p;q;r];
% states = [u;w;q];
states = [v;p;r];

% controls = [delta_e;delta_a;delta_t];
% controls = [delta_e;delta_t];
controls = delta_a;

C_L_0 = MX.sym('C_L_0');
C_L_alpha = MX.sym('C_L_alpha');
C_L_delta_e = MX.sym('C_L_delta_e');
C_L_q = MX.sym('C_L_q');
C_D_0 = MX.sym('C_D_0');
C_D_alpha1 = MX.sym('C_D_alpha1');
C_D_delta_e = MX.sym('C_D_delta_e');
C_D_q = MX.sym('C_D_q');
C_m_0 = MX.sym('C_m_0');
C_m_alpha = MX.sym('C_m_alpha');
C_m_delta_e = MX.sym('C_m_delta_e');
C_m_q = MX.sym('C_m_q');
C_Y_0 = MX.sym('C_Y_0');
C_Y_beta = MX.sym('C_Y_beta');
C_Y_delta_a = MX.sym('C_Y_delta_a');
C_Y_p = MX.sym('C_Y_p');
C_Y_r = MX.sym('C_Y_r');
C_l_0 = MX.sym('C_l_0');
C_l_beta = MX.sym('C_l_beta');
C_l_delta_a = MX.sym('C_l_delta_a');
C_l_p = MX.sym('C_l_p');
C_l_r = MX.sym('C_l_r');
C_n_0 = MX.sym('C_n_0');
C_n_beta = MX.sym('C_n_beta');
C_n_delta_a = MX.sym('C_n_delta_a');
C_n_p = MX.sym('C_n_p');
C_n_r = MX.sym('C_n_r');

params_lon = [C_L_0;C_L_alpha;C_L_delta_e;C_L_q;C_D_0;C_D_alpha1;...
     C_D_delta_e;C_D_q;C_m_alpha;C_m_delta_e;C_m_q;C_m_0];

params_lat = [C_Y_p;C_Y_r;C_Y_0;C_Y_beta;C_Y_delta_a;C_l_0;C_l_beta;...
    C_l_p;C_l_r;C_l_delta_a;C_n_0;C_n_beta;C_n_p;C_n_r;C_n_delta_a];

% params = params_lon;
params = params_lat;
sysparam = params;


% xdot = eom_longitudinal_sysid(states,controls,x(5),x(7),x(8),x(10),x(12),P,sysparam);
xdot = eom_lateral_sysid(states,controls,x(4),x(6),x(7),x(8),x(11),P,sysparam);
rhs = xdot;

% Form an ode function
ode = Function('ode',{states,controls,params},{rhs});

%%%%%%%%%%%% Creating a simulator %%%%%%%%%%
N_steps_per_sample = 1;
dt = 1/fs/N_steps_per_sample;

% Build an integrator for this system: Runge Kutta 4 integrator
k1 = ode(states,controls,params);
k2 = ode(states+dt/2.0*k1,controls,params);
k3 = ode(states+dt/2.0*k2,controls,params);
k4 = ode(states+dt*k3,controls,params);

states_final = states+dt/6.0*(k1+2*k2+2*k3+k4);

% Create a function that simulates one step propagation in a sample
one_step = Function('one_step',{states, controls, params},{states_final});

X = states;
for i=1:N_steps_per_sample
    X = one_step(X, controls, params);
end

% Create a function that simulates all step propagation on a sample
one_sample = Function('one_sample',{states, controls, params}, {X});

% speedup trick: expand into scalar operations
one_sample = one_sample.expand();

%%%%%%%%%%%% Simulating the system %%%%%%%%%%

all_samples = one_sample.mapaccum('all_samples', N);

% % Choose an excitation signal
% u_data = 0.1*rand(N,1);

% x0 = DM(x0_lon);
x0 = DM(x0_lat);

% y_data = x_lon';
y_data = x_lat';

%%%% Identifying the simulated system: single shooting strategy %%%%

% Note, it is in general a good idea to scale your decision variables such
% that they are in the order of ~0.1..100
% X_symbolic = all_samples(x0, u_lon(1:200,:)', repmat(params.*scale,1,N));
X_symbolic = all_samples(x0, u_lat(1:200,:)', repmat(params.*scale,1,N));

e = y_data(:,1:200)-X_symbolic;

nlp = struct;
nlp.x = params;
nlp.f = 0.5*dot(e,e);
opts = struct;
opts.ipopt.print_level = 0;
% opts.ipopt.max_iter = 300;

solver = nlpsol('solver','ipopt',nlp,opts);
sol = solver('x0',param_guess);
sol_x = full(sol.x).*scale;
disp(full(sol.x).*scale)

save('sol_x.mat','sol_x')
%%
figure(1)
x0=700;
y0=500;
width=1000;
height=500;
set(gcf,'position',[x0,y0,width,height])
xaxis = [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15];
% names = {'C_{L,0}'; 'C_{L,alpha}'; 'C_{L,delta_e}'; 'C_{L,q}'; ...
%     'C_{D,0}'; 'C_{D,alpha1}'; 'C_{D,delta_e}'; 'C_{D,q}'; 'C_{m,alpha}'; ...
%     'C_{m,delta_e}'; 'C_{m,q}'; 'C_{m_0}'};
names = {'C_{Y,p}'; 'C_{Y,r}'; 'C_{Y,0}'; 'C_{Y,beta}'; 'C_{Y,delta_a}'; ...
    'C_{l,0}'; 'C_{l,beta}'; 'C_{l,p}'; 'C_{l,r}'; 'C_{l,delta_a}'; ...
    'C_{n,0}'; 'C_{n,beta}'; 'C_{n,p}'; 'C_{n,r}'; 'C_{n,delta_a}'};

plot(xaxis,100*abs((param_truth-sol_x)./param_truth),'*','Linewidth',10)
set(gca,'xtick',[1:15],'xticklabel',names,'Fontsize',14)
ylabel('Percent error [%]');
title('System identification results')
