function [xtrim,sol] = trim(Va_d, R_d, gamma_d, x0, P)
%Returns trim state and control input for desired airspeed, curve radius and flight-path angle.
%returns: x_trim = [pos, vel, Euler, ang_vel], u_trim = [delta_e, delta_a, delta_t]
import casadi.*

x = SX.sym('x', 15, 1);

x_dot_d = [0, 0, -Va_d*sin(gamma_d), 0, 0, 0, 0, 0, Va_d*cos(gamma_d)/R_d, 0, 0, 0]';
statespace = 'full';
x_dot = equationsOfMotion(x(1:12),x(13:15),P,statespace);

sum = 0;
N = 12;
for i = 3:N
    sum = sum + (x_dot_d(i) - x_dot(i))^2;
end

f = sum;
nlp = struct;            % NLP declaration
nlp.x = x;               % decision vars
nlp.f = f;               % objective
lbx = [-Inf,-Inf,-Inf,-Inf,-Inf,-Inf,-Inf,-Inf,-Inf,-Inf,-Inf,-Inf,-0.52,-0.52,0]';
ubx = [Inf,Inf,Inf,Inf,Inf,Inf,Inf,Inf,Inf,Inf,Inf,Inf,0.52,0.52,1]';

opts = struct;
opts.ipopt.print_level = 0;

% Create solver instance
solver = nlpsol('solver','ipopt',nlp,opts);
sol = solver('x0',x0,'lbx',lbx,'ubx',ubx);
xtrim = full(sol.x);
disp('Trim computation complete')
end
